## Distributed as part of the supplementary material for the the paper
## ``Improving the accuracy of likelihood-based inference in
## meta-analysis and meta-regression''
##
## Authors: Ioannis Kosmidis, Annamaria Guolo, Cristiano Varin
## Date: 15 October 2016
## Licence: GPL 2 or greater

## NOT A POLISHED PIECE OF PUBLIC-USE SOFTWARE!  Provided "as is".
## NO WARRANTY OF FITNESS FOR ANY PURPOSE!

## Simulate Standardized Mean Differences
##
## Arguments:
## num.studies  number of studies
## min.size     minimum number of subjects per study
## max.size     maximum number of subjects per study
## proportion   ratio between per study number of subjects into two arms
## delta        the 'true' standardized mean difference
## var.re       variance of the random effect
## Hedges       whether the Hedges' correction is applied or notο

simulate.SMD <- function(num.studies = 5,
                         min.size = 30,
                         max.size = 120,
                         proportion = 1, ## balanced study
                         delta = -2,
                         var.re = 2,
                         Hedges = TRUE)
{

    mu1 <- 0.0 ## arbitrary
    sigma <- 1.0 ## arbitrary

    sizes1 <- sample(min.size : max.size, num.studies, replace = TRUE)
    sizes2 <- round(proportion * sizes1)

    ## Hedges's g statistic
    g <- se.g <- rep(NA, num.studies)

    for(i in 1:num.studies){

        ## mean arm 2
        mu2 <- (delta + sqrt(var.re) * rnorm(1) ) * sigma + mu1
        ## simulation two arms
        nobs1 <- sizes1[i]
        nobs2 <- sizes2[i]
        arm1 <- rnorm(nobs1, mean = mu1, sd = sigma)
        arm2 <- rnorm(nobs2, mean = mu2, sd = sigma)

        ## pooled variance
        s2pooled <- ((nobs1 - 1) * var(arm1) + (nobs2 - 1) * var(arm2)) / (nobs1 + nobs2 - 2)
        ## compute SMD
        J <- 1
        if(Hedges) J <- 1 - 3 / (4 * (nobs1 + nobs2 - 2) - 1)
        delta.hat <- (mean(arm2) - mean(arm1)) / sqrt(s2pooled)
        var.delta.hat <- (nobs1 + nobs2) / (nobs1 * nobs2) + delta.hat ^ 2 / (2 * (nobs1 + nobs2))
        ## compute Hedges' g
        g[i] <- J * delta.hat
        se.g[i] <- J * sqrt(var.delta.hat)
    }

    return(list(betahat = g, betahat.se = se.g))
}
